

#include <Process.au3>
#Include <Array.au3>

#cs

GlossLookup looks up words in a sentence in a glossary, and looks up glossary terms in the same sentence.  Essentially, the script does two separate types of searches, one after the other.  In the first search, it checks to see if any of the whole source text of any glossary entries occurs in the sentence.  In the second search, it checks if any of the words in the sentence occur in the source text of all glossary entries.  The results are output to HTML, then opened in a browser.

It may be possible to add stemming later, by simply deleting the stems (from long to short) from the compared text before making the comparision.

#ce

#cs

version 2b has ctrlenter function added, has cleaner layout, and asks if to restore focus to idiom

#ce


Global $fileopenhtml, $glossaryfilecolumns, $glossaryfilelines, $glossaryfileopen, $glossaryfileopendialog, $glossaryfileread, $grabbedtext, $grabbedword, $grabbedwordarray, $htmloutput, $i, $j, $k, $l, $output, $restorefocus

MsgBox (0, "GlossLookup version 2b", "GlossLookup looks up words from a segment in a glossary.  The results are opened in a web browser." & @CRLF & @CRLF & "Shortcut:" & @CRLF & "To check the current segment, press F4." & @CRLF & "To move to the next segment and automatically check it, press Ctrl+Enter." & @CRLF & "To exit the program, press WinKey + Q", 0)

$restorefocus = MsgBox (4, "Restore focus to Idiom after lookup?", "Would you like to restore the focus to Idiom after two seconds after the output was displayed in the browser?" & @CRLF & @CRLF & "Say 'yes' if you have enough space on your screen to show Idiom and the browser windows side-by-side (not overlapping each other).  It doesn't always work, if you have a slow browser.", 0)


; ==================
; Step 1: define hotkeys
; ==================

; HotKeySet("#c", "grabclipandgo")
HotKeySet("^{ENTER}", "ctrlenter")
HotKeySet("{F4}", "copyallandthengrabclipandgo")
HotKeySet("#q", "exitty")

; =======================
; Step 2: add glossary to array
; =======================

#cs
$glossaryfileopendialog = FileOpenDialog ("Select glossary", @ScriptDir, "Text files (*.txt)|All files (*.*)") ; uncomment this to be asked for glossary at start of run
#ce

; #cs
$glossaryfileopendialog = @ScriptDir & "\glossary.txt" ; comment this if you uncomment the above
; #ce

$glossaryfileopen = FileOpen ($glossaryfileopendialog, 128) ; 128 means UTF8, 32 means UTF16LE
$glossaryfileread = FileRead ($glossaryfileopen)

$glossaryfilelines = StringSplit ($glossaryfileread, @CRLF, 3)

Global $glossaryfilefirstcolumn[UBound ($glossaryfilelines)]

For $i = 0 to UBound ($glossaryfilelines) - 1
$glossaryfilecolumns = StringSplit ($glossaryfilelines[$i], @TAB, 3)
If UBound ($glossaryfilecolumns) > 1 Then ; i.e. if there are at least 2 columns in the line
$glossaryfilefirstcolumn[$i] = $glossaryfilecolumns[0]
EndIf
Next

; MsgBox (0, "", _ArrayToString($glossaryfilefirstcolumn, " | "), 0) ; If you want to check that the entire glossary was read

; ========================
; Step 3: let script stay resident
; ========================

While 1
Sleep ("100")
WEnd

; =======================================
; Step 3: define functions that respond to hotkeys
; =======================================

Func exitty ()
Exit
EndFunc

Func copyallandthengrabclipandgo()

Send ("{F5}")
Sleep ("200")
Send ("^a")
Sleep ("200")
Send ("^x")
Sleep ("200")
Send ("^z")
Sleep ("200")
grabclipandgo()

EndFunc

Func grabclipandgo()

$grabbedtext = ClipGet()

$output = ""

; ==================
; DO TERM MATCHING
; ==================

$output = $output & "<br><b><u>TERM MATCHING</u></b><p><br>"

For $m = 0 to UBound ($glossaryfilelines) - 1

; #cs ; ugly hack to prevent too many false positives for short terms
If StringLen ($glossaryfilefirstcolumn[$m]) < 4 Then
$glossaryfilefirstcolumn[$m] = " " & $glossaryfilefirstcolumn[$m] & " "
EndIf
; #ce

If StringInStr ($grabbedtext, $glossaryfilefirstcolumn[$m]) Then
$output = $output & "<b>" & $glossaryfilefirstcolumn[$m] & "</b><br>" & $glossaryfilelines[$m] & "<p><br>"
EndIf

Next

; ==================
; DO WORD MATCHING
; ==================

$grabbedword = StringSplit ($grabbedtext, " .,!?<>{}[]()@#$&", 2)

$output = $output & "<br><b><u>WORD MATCHING</u></b><p><br>"

For $k = 0 to UBound ($grabbedword) -1
For $l = 0 to UBound ($glossaryfilelines) - 1

If StringLen ($grabbedword[$k]) > 4 Then ; don't do word matching for short words

; #cs
If $grabbedword[$k] = $glossaryfilefirstcolumn[$l] Then ; If grabbed word equals glossary term ; comment this out if you don't prefer exact matching -- less accurate, more matches
; #ce

#cs
If StringInStr ($glossaryfilefirstcolumn[$l], $grabbedword[$k]) Then ; uncomment this if you comment the above
#ce

$output = $output & "<b>" & $grabbedword[$k] & "</b><br>" & $glossaryfilelines[$l] & "<p><br>"

EndIf
EndIf

Next
Next

; ================================
; DO SOME HOUSEKEEPING ON OUTPUT
; ================================

$htmloutput = StringReplace ($output, @TAB, "<br>")

While StringInStr ($htmloutput, "<br><br>")
$htmloutput = StringReplace ($htmloutput, "<br><br>", "<br>")
WEnd

While StringInStr ($htmloutput, "<br> <br>")
$htmloutput = StringReplace ($htmloutput, "<br> <br>", "<br>")
WEnd

While StringInStr ($htmloutput, "<br>  <br>")
$htmloutput = StringReplace ($htmloutput, "<br>  <br>", "<br>")
WEnd

$htmloutput = StringReplace ($htmloutput, @CRLF, "<p>") ; remnant from old version

; ====================================
; CREATE HTML FILE AND OPEN IN BROWSER
; ====================================

$fileopenhtml = FileOpen ("output.html", 130) ; 130 means overwrite if exists, 34 would mean the same for UTF16LE

FileWrite ($fileopenhtml, '<html><head><title>' & $grabbedtext & '</title></head><body><table width="100%" align="center" cellpadding="10" cellspacing="0" border="1"><tr><td><b> ' & $grabbedtext & '</b></td></tr></table><br>' & $htmloutput & '</body></html>')

FileClose ($fileopenhtml)

Run(@ComSpec & " /c " & 'start output.html', "", @SW_HIDE)

; $grabbedword =""
; $grabbedtext =""
$output = "" ; set output back to nothing

If $restorefocus = 6 then
; WinWaitActive ($grabbedtext, "") ; no this is dangerous
Sleep ("2500")
WinActivate ("Idiom", "")
EndIf

EndFunc

Func ctrlenter()

Send ("{ENTER}")
Sleep ("200")
copyallandthengrabclipandgo()
EndFunc